<?php
/**
 * 2007-2021 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    PrestaShop SA <contact@prestashop.com>
 *  @copyright 2007-2021 PrestaShop SA
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

class PaymentLock
{
    const TABLE_PAYMENT_LOCK = '_payment_lock';

    /**
     * Must be the same as module name
     * @see Montonio::name
     *
     * @var string
     */
    public $name = 'montonio';

    /**
     * @throws Exception
     */
    public function installResponseLockTable()
    {
        $db = Db::getInstance(true);
        if (!$db->execute('
                CREATE TABLE IF NOT EXISTS `' . $this->getTable(self::TABLE_PAYMENT_LOCK) . '` (
                    `id_cart`	INT UNSIGNED		NOT NULL,
                    `lock`		CHAR(23)			NOT NULL,
                    PRIMARY KEY (`id_cart`)
                );', false)
        ) {
            throw new Exception(
                sprintf(
                    $this->l('Fatal error: Installation of the db table failed, error code: %u, error message: %s'),
                    $db->getNumberError(),
                    $db->getMsgError()
                )
            );
        }
    }

    public function getResponseLock($id_cart)
    {
        $db     = Db::getInstance(true);
        $result = $db->getValue('SELECT `lock` from `' . $this->getTable(self::TABLE_PAYMENT_LOCK)
            . '` WHERE `id_cart` = ' . (int) $id_cart, false);

        return $result;
    }

    public function tryCreateResponseLock($id_cart, &$lock = null)
    {
        if (is_null($lock)) {
            $lock = uniqid('', true);
        }

        $db = Db::getInstance(true);
        $db->execute('
            INSERT IGNORE INTO `' . $this->getTable(self::TABLE_PAYMENT_LOCK) . '`
            SET
                `id_cart` = ' . (int) $id_cart . ',
                `lock` = \'' . pSQL($lock) . '\'
            ;
        ', false);

        return $lock == $this->getResponseLock($id_cart);
    }

    public function removeResponseLock($id_cart, $lock)
    {
        $db = Db::getInstance(true);
        $db->execute('
            DELETE FROM `' . $this->getTable(self::TABLE_PAYMENT_LOCK) . '`
            WHERE
                `id_cart` = ' . (int) $id_cart . ' AND
                `lock` = \'' . pSQL($lock) . '\'
            ;
        ', false);
    }

    public function getTable($table, $addPrefix = true)
    {
        return ($addPrefix ? _DB_PREFIX_ : '') . $this->name . $table;
    }
}
