<?php

/**
 * This class controls the UI side of the Checkout page
 */
class Montonio_Payments_Checkout
{
    protected $_payment_handle_style;
    protected $_payment_handle_extra_css;
    protected $_payment_logos_list;
    protected $_preferred_country;
    protected $_is_non_default_description = false;
    protected $_attempt_to_select_payment = false;
    protected $regions = [
        'EE' => 'Estonia',
        'LT' => 'Lithuania',
        'LV' => 'Latvia',
        'FI' => 'Finland',
        'PL' => 'Poland',
        'DE' => 'Germany'
     ];
    
    /**
     * The description shown at checkout
     *
     * @var string
     */
    protected $_description;
    
    /**
     * Banks to show at checkout
     * (configured in Montonio_Payments_Settings)
     *
     * @var Object
     */
    protected $_bankList;
    
    public function get_description_html()
    {

        $bankList = $this->_bankList['paymentMethods']['paymentInitiation']['setup'] ?? [];

        if (empty($bankList)) {
            return $this->get_default_description();
        }

        $idCurrency      = Context::getContext()->cart->id_currency;
        $paymentCurrency = new Currency($idCurrency); // should contain iso_code

        if (!isset($paymentCurrency->iso_code)) {
            return $this->get_default_description();
        }

        $currency       = $paymentCurrency->iso_code;

        $paymentMethods = $this->filter_payment_methods_by_currency($bankList, $currency);

        switch ($this->get_payment_handle_style()) {
            case 'list_logos':
                return $this->get_html_list_logos($paymentMethods, $currency  );
            case 'grid_logos':

                return $this->get_html_grid_logos($paymentMethods, $currency  );
            default:
                return $this->get_default_description();
        }
    }
    
    // =========================================================================
    // Style and script for non-default description
    // =========================================================================
    
    public function register_payment_handle_style()
    {
        if ($this->_is_non_default_description) {
           
        }
    }

    public function register_payment_handle_script()
    {
      
    }
    
    // =========================================================================
    // Checkout HTML types
    // =========================================================================
    
    protected function get_default_description()
    {
        return $this->get_description(); // TODO: Translations
    }
    
    protected function get_html_list_logos($regions, $currency)
    {
        $preselectedAspsp = '';
        $description = (count($regions) > 1) ? $this->get_dropdown_html($regions) : '';
        $description .= '<ul id="montonio-payments-description" class="montonio-aspsp-ul montonio-aspsp-list-logos">';

        $defaultCountryToDisplay = array_keys($regions)[0];
        foreach ($regions as $region => $value) {
            if ($region === $this->get_preferred_country()) {
                $defaultCountryToDisplay = $region;
            }
        }
        
        foreach ($regions as $r => $list) {
            foreach ($list['paymentMethods'] as $key => $value) {
                if(in_array($currency,$value['supportedCurrencies'])) {
                    $description .= '<li data-aspsp="' . $value['identifer'] . '" class="aspsp-region-'. $r .' montonio-aspsp-li montonio-aspsp '. ($r == $defaultCountryToDisplay ? '' : 'montonio-hidden') .'"><img class="montonio-aspsp-li-img" src="' . $value['logoUrl'] . '"></li>';
                }
            }
        }
        $description .= '</ul>';
        $this->_is_non_default_description = true;
        return $preselectedAspsp . $description;
    }
    
    protected function get_html_grid_logos($regions, $currency)
    {

        $preselectedAspsp = '';
        $description = (count($regions) > 1) ? $this->get_dropdown_html($regions) : '';
        $description .= '<div id="montonio-payments-description" class="montonio-aspsp-grid montonio-aspsp-grid-logos">';

        $defaultCountryToDisplay = current(array_keys($regions));
        foreach ($regions as $region => $value) {
            if ($region === $this->get_preferred_country()) {
                $defaultCountryToDisplay = $region;
            }
        }

        foreach ($regions as $r => $list) {

            foreach ($list['paymentMethods'] as $value) {
                if(in_array($currency,$value['supportedCurrencies'])) {
                    $description .= '<div class="aspsp-region-' . $r . ' montonio-aspsp-grid-item montonio-aspsp ' . ($r == $defaultCountryToDisplay ? '' : 'montonio-hidden') . '" data-aspsp="' . $value['identifer']
                        . '"><img class="montonio-aspsp-grid-item-img" src="' . $value['logoUrl'] . '"></div>';
                }
            }
        }
        $description .= '</div>';
            
        $this->_is_non_default_description = true;
        return $preselectedAspsp . $description;
    }

    protected function get_dropdown_html($regions) {
        $attempt_to_select = $this->get_attempt_to_select_payment_html();
        $html =  $attempt_to_select . '<select id="montonio-payments-country-dropdown" class="montonio-payments-country-dropdown" name="country">';
        foreach ($regions as $r => $list) {
            $html .= '<option '. ($r == $this->get_preferred_country() ? 'selected="selected"' : '') .' value="'. $r .'">'. $this->regions[$r] .'</option>';
        }
        $html .= '</select>';
        // $html .= '<script src="' . MONTONIO_PLUGIN_URL . 'payments/assets/js/montonio-payment-handle.js"></script>';

        return $html;
    }

    protected function get_attempt_to_select_payment_html() {
        return '<input type="hidden" id="montonio_attempt_to_select_payment_method" value="'. ($this->get_attempt_to_select_payment() ? 'yes' : 'no') .'">';
    }

    protected function filter_payment_methods_by_currency($payment_methods, $currency)
    {
        return array_filter($payment_methods, function($country) use ($currency) {
            return array_search($currency, $country['supportedCurrencies']) !== false;
        });
    }
    
    // =========================================================================
    // Getters and setters
    // =========================================================================
    
    /**
     * @return object
     */
    public function get_payment_handle_style()
    {
        return $this->_payment_handle_style;
    }
    
    /**
     * @param object $payment_handle_style
     */
    public function set_payment_handle_style($payment_handle_style)
    {
        $this->_payment_handle_style = $payment_handle_style;
    }
    
    /**
     * @return object
     */
    public function get_payment_handle_extra_css()
    {
        return $this->_payment_handle_extra_css;
    }
    
    /**
     * @param object $payment_handle_extra_css
     */
    public function set_payment_handle_extra_css($payment_handle_extra_css)
    {
        $this->_payment_handle_extra_css = $payment_handle_extra_css;
    }
    
    /**
     * JSON-decode bankList
     *
     * @param object $payment_handle_style
     */
    public function set_banklist($banklistJson)
    {
        $this->_bankList = json_decode($banklistJson, true);
    }
    
    public function set_description($description) {
        $this->_description = $description;
    }
    
    public function get_description() {
        return $this->_description;
    }

    public function set_preferred_country($country) {
        $this->_preferred_country = $country;
    }

    public function get_preferred_country() {
        return $this->_preferred_country;
    }

    public function set_regions($regions) {
        $this->regions = $regions;
    }

    public function get_regions() {
        return $this->regions;
    }

    public function set_attempt_to_select_payment($attempt_to_select_payment) {
        $this->_attempt_to_select_payment = $attempt_to_select_payment == 'yes';
    }

    public function get_attempt_to_select_payment() {
        return $this->_attempt_to_select_payment;
    }
}